<?php
/*
Plugin Name: PayPal File Download
Plugin URI: http://yesplugins.com/paypal_file_download
Description: PayPal File Download lets you easily set up a PayPal checkout system that delivers a file to a user, with integrated, transparent, and full-featured IPN, email delivery, link expiry, customer information capture, and more.
Version: 1.3
Author: Mark Bao / YesPlugins
Author URI: http://yesplugins.com/
License: GPL2
*/

PayPalFileDownload::init();

class PayPalFileDownload {
	protected static $currencies = array(
		'USD' => array('United States Dollar','$'),
		'AUD' => array('Australian Dollar','AUD$'),	
		'BRL' => array('Brazilian Real','R$'),
		'GBP' => array('British Pound','&pound;'),
		'CAD' => array('Canadian Dollar','CAD$'),
		'CNY' => array('Chinese Yuan','&#20803;'),
		'DKK' => array('Danish Krone','kr.'),
		'EUR' => array('European Euro','&#8364;'),
		'HKD' => array('Hong Kong Dollar','HK$'),
		'HUF' => array('Hungarian Forint','Ft'),
		'INR' => array('Indian Rupee','INR'),
		'IDR' => array('Indonesian Rupiah','Rp'),
		'JPY' => array('Japanese Yen','&yen;'),
		'MXN' => array('Mexican Peso','MEX$'),
		'NZD' => array('New Zealand Dollar','NZ$'),
		'NOK' => array('Norwegian Kroner','kr'),
		'PLN' => array('Polish Zloty','zl.'),
		'RUB' => array('Russian Ruble','RUB'),
		'SAR' => array('Saudi Riyal','SR'),
		'SGD' => array('Singapore Dollar','SGD$'),
		'ZAR' => array('South African Rand','R'),
		'SEK' => array('Swedish Krona','kr'),
		'CHF' => array('Swiss Franc','CHF'),
		'THB' => array('Thai Bhat','&#3647;'),
		'TRY' => array('Turkish Lira','TRY'),
		'TWD' => array('Taiwan Dollar','TWD')
	);

	const VERSION = '1.3';
	const DB_VERSION = "1.0";

	public static function init() {
		register_activation_hook(__FILE__, array(__CLASS__, 'install'));

		// admin stuff
		add_action('admin_menu', array(__CLASS__, 'admin_menu'));
		add_action('admin_init', array(__CLASS__, 'admin_init'));

		// media buttons hook
		add_action('media_buttons_context', array(__CLASS__, 'media_button'));

		// insert form
		add_action('admin_footer', array(__CLASS__, 'add_pfd_form'));

		// listener for ipn activation
		add_action('template_redirect', array(__CLASS__, 'var_listener'));
		add_filter('query_vars', array(__CLASS__, 'register_vars'));
		
		add_action('admin_menu', array(__CLASS__, 'add_meta_box'));
	}

	protected static function transactioncode($length = "") {
		$code = md5(uniqid(rand(), true));
		if ($length != "") return strtoupper(substr($code, 0, $length));
		else return strtoupper($code);
	}
 
	protected static function relative_time($ptime) {
		$etime = time() - $ptime;

		if ($etime < 1) {
			return '0 seconds';
		}

		$a = array( 12 * 30 * 24 * 60 * 60  =>  'year',
					30 * 24 * 60 * 60       =>  'month',
					24 * 60 * 60            =>  'day',
					60 * 60                 =>  'hour',
					60                      =>  'minute',
					1                       =>  'second'
					);

		foreach ($a as $secs => $str) {
			$d = $etime / $secs;
			if ($d >= 1) {
				$r = round($d);
				return $r . ' ' . $str . ($r > 1 ? 's' : '');
			}
		}
	}
	
	public static function add_meta_box() {
		add_meta_box( 'pfd_sectionid', "PayPal File Download", array(__CLASS__, 'meta_box'), 'post', 'side', 'high' );
		add_meta_box( 'pfd_sectionid', "PayPal File Download", array(__CLASS__, 'meta_box'), 'page', 'side', 'high' );
	}
	
	public static function meta_box() {
//		echo '<div style="margin-top:5px;margin-bottom:5px;">';
		echo '<a href="#TB_inline?width=450&inlineId=paypal_file_download_form" class="thickbox button" title="Insert PayPal Checkout Link">Insert PayPal Buy Button/Link</a>';
//		echo '</div>';
	}

	public static function install() {
		global $wpdb;
		
		$message_default = <<<EOT
Hi [FIRST_NAME] [LAST_NAME],

Thank you for your purchase of [PRODUCT_NAME]! Your download link is included at the bottom of this message. For future reference, your transaction ID is [TRANSACTION_ID].

Thank you, and enjoy!
EOT;

		$message_default_nofile = <<<EOT
Hi [FIRST_NAME] [LAST_NAME],

Thank you for your purchase of [PRODUCT_NAME]! For future reference, your transaction ID is [TRANSACTION_ID].

Thank you, and enjoy!
EOT;
		
		add_option("email_message", $message_default, '','yes');
		add_option("email_message_nofile", $message_default_nofile, '','yes');
		add_option("expire_links_after", 7, '','yes');
		add_option("paypal_email", "test@replaceme.com", '','yes');
		add_option("paypal_return_url", get_option("siteurl"), '','yes');

		$table_name = $wpdb->prefix . "pfd_products";
		if($wpdb->get_var("show tables like '$table_name'") != $table_name) {
			$sql = "CREATE TABLE " . $table_name . " (
				id mediumint(9) NOT NULL AUTO_INCREMENT,
				name VARCHAR(255) NOT NULL,
				file VARCHAR(255) NOT NULL,
				downloads bigint(11) NOT NULL,
				cost decimal(11,2) NOT NULL,
				created_at bigint(11) DEFAULT '0' NOT NULL,
				PRIMARY KEY id (id)
			);";

			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
		
		$table_name = $wpdb->prefix . "pfd_orders";
		if($wpdb->get_var("show tables like '$table_name'") != $table_name) {
			$sql = "CREATE TABLE " . $table_name . " (
				id mediumint(9) NOT NULL AUTO_INCREMENT,
				product_id mediumint(9) NOT NULL,
				order_code VARCHAR(255) NOT NULL,
				fulfilled mediumint(9) NOT NULL,
				cost decimal(11,2) NOT NULL,
				created_at bigint(11) DEFAULT '0' NOT NULL,
				PRIMARY KEY id (id)
			);";

			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
		
		
		$table_name = $wpdb->prefix . "pfd_transactions";
		if($wpdb->get_var("show tables like '$table_name'") != $table_name) {
			$sql = "CREATE TABLE " . $table_name . " (
				id mediumint(9) NOT NULL AUTO_INCREMENT,
				product_id mediumint(9) NOT NULL,
				order_id mediumint(9) NOT NULL,
				order_code VARCHAR(255) NOT NULL,
				protection_eligibility VARCHAR(255) NULL,
				address_status VARCHAR(255) NULL,
				payer_id VARCHAR(255) NULL,
				tax decimal(11,2) NULL,
				payment_date VARCHAR(255) NULL,
				payment_status VARCHAR(255) NULL,
				first_name VARCHAR(255) NULL,
				last_name VARCHAR(255) NULL,
				payer_status VARCHAR(255) NULL,
				business VARCHAR(255) NULL,
				address_name VARCHAR(255) NULL,
				address_street VARCHAR(255) NULL,
				address_city VARCHAR(255) NULL,
				address_state VARCHAR(255) NULL,
				address_zip VARCHAR(255) NULL,
				address_country_code VARCHAR(255) NULL,
				address_country VARCHAR(255) NULL,
				quantity VARCHAR(255) NULL,
				verify_sign VARCHAR(255) NULL,
				payer_email VARCHAR(255) NULL,
				txn_id VARCHAR(255) NULL,
				payment_type VARCHAR(255) NULL,
				receiver_email VARCHAR(255) NULL,
				receiver_id VARCHAR(255) NULL,
				txn_type VARCHAR(255) NULL,
				item_name VARCHAR(255) NULL,
				mc_currency VARCHAR(255) NULL,
				item_number VARCHAR(255) NULL,
				residence_country VARCHAR(255) NULL,
				custom VARCHAR(255) NULL,
				receipt_id VARCHAR(255) NULL,
				transaction_subject VARCHAR(255) NULL,
				payment_fee decimal(11,2) NOT NULL,
				payment_gross decimal(11,2) NOT NULL,
				created_at bigint(11) DEFAULT '0' NOT NULL,
				PRIMARY KEY id (id)
			);";

			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
		
		add_option("pfd_db_version", self::DB_VERSION);
	}

	protected static function get_currency() {
		if (get_option('pfd_currency')) {
			$cc = get_option('pfd_currency');
		} else {
			$cc = "USD";
		}
		
		return $cc;
	}

	protected static function get_currency_symbol() {
		$cc = self::get_currency();
		
		return self::$currencies[$cc][1];
	}

	public static function validate_currency($currency) {
		if (!empty(self::$currencies[$currency]))
			return $currency;
		return 'USD';
	}

	public static function admin_init() {
		register_setting('pfd_options', 'email_message');
		register_setting('pfd_options', 'email_message_nofile');
		register_setting('pfd_options', 'expire_links_after', 'intval');
		register_setting('pfd_options', 'paypal_email');
		register_setting('pfd_options', 'paypal_return_url');
		register_setting('pfd_options', 'pfd_currency', array(__CLASS__, 'validate_currency'));
	}

	public static function admin_menu() {
		add_menu_page( "PayPal File Download", "PayPal File Download", 'manage_options', 'paypal-file-download', array(__CLASS__, 'admin_dashboard'), get_option('siteurl') . "/wp-content/plugins/paypal_file_download/menu_icon.png");

		add_submenu_page( 'paypal-file-download', "PayPal File Download Products", "Products", 'manage_options', "paypal-file-download-products", array(__CLASS__, 'admin_products_router'));

		add_submenu_page( 'paypal-file-download', "PayPal File Download Settings", "Settings", 'manage_options', "paypal-file-download-settings", array(__CLASS__, 'admin_settings'));

		add_submenu_page( 'paypal-file-download', "PayPal File Download Transactions", "Transactions", 'manage_options', "paypal-file-download-transactions", array(__CLASS__,'admin_transactions'));

		global $submenu;
		$submenu['paypal-file-download'][0][0] = 'Support';
	}

	public static function admin_products_router() {
		$action = '';
		if (!empty($_REQUEST['action'])) {
			$action = $_REQUEST['action'];
		}
		
		switch ($action) {
			case 'edit':
				return self::admin_products_edit();
				break;
			case 'delete':
				return self::admin_products_delete();
				break;
			case 'add':
				return self::admin_products_add();
				break;
			default:
				return self::admin_products();
		}
	}

	protected static function admin_products_edit() {
		
		global $wpdb;
		$table_name = $wpdb->prefix . "pfd_products";
		
		if (isset($_POST["product_name"])) {
		
			$name = $_POST["product_name"];
			$url = $_POST["product_url"];
			$cost = $_POST["product_cost"];
			
			$wpdb->update( $table_name, array('name' => $name, 'file' => $url, 'cost' => $cost), array('id' => $_GET["id"]), array( '%s', '%s', '%s'));
		}
		
		$product = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE id = %d",$_GET["id"]) , ARRAY_A, 0);
	?>
	<div class="wrap">
		<h2>Edit Product: <?php echo $product['name'] ?></h2>
		<a href="<?php echo get_option('siteurl'); ?>/wp-admin/admin.php?page=paypal-file-download-products">&laquo; Back to Products</a>
		<form action="<?php echo get_option('siteurl'); ?>/wp-admin/admin.php?page=paypal-file-download-products&action=edit&id=<?php echo $_GET['id'] ?>" method="post">
		<table class="form-table">
			<tr valign="top">
				<th scope="row">Product Name</th>
				<td><input type="text" name="product_name" style="width:250px;" value="<?php echo str_replace('"','\"',$product["name"]); ?>" /></td>
			</tr>
			<tr valign="top">
				<th scope="row">URL of Product</th>
				<td><input type="text" name="product_url" style="width:400px;" value="<?php echo str_replace('"','\"',$product["file"]); ?>" /><br />(make sure it's pretty secret<br />This is never shown to the user. Instead, the file is streamed from this location directly to the user through a secret page, which helps to prevent content piracy.)</td>
			</tr>
			<tr valign="top">
				<th scope="row">Cost Per Download</th>
				<td><?php echo self::get_currency_symbol() ?><input type="text" name="product_cost" style="width:50px;" value="<?php echo str_replace('"','\"',$product["cost"]); ?>" /></td>
			</tr>
			<tr valign="top">
				<th scope="row">&nbsp;</th>
				<td>
					<input type="submit" class="button-primary" value="Save Product" />
				</td>
			</tr>
		</table>
		</form>
	</div>
	<?php
	}

	protected static function admin_products_delete() {
		// delete and redirect
		global $wpdb;
		$table_name = $wpdb->prefix . "pfd_products";
		
		$id = $_GET["id"];
		
		$wpdb->query("DELETE FROM $table_name WHERE id = '$id'");
		?>
		<script type="text/javascript">
		<!--
		window.location = "<?php echo get_option('siteurl') . '/wp-admin/admin.php?page=paypal-file-download-products' ?>"
		//-->
		</script>
		<?php
	}

	public static function admin_dashboard() {
		if (!current_user_can('manage_options'))  {
			wp_die( __('You do not have sufficient permissions to access this page.') );
		}
		
	?>	
	<div class="wrap">
		<iframe src="http://backend.yesplugins.com/dash/show/?plugin_id=paypal_file_download&plugin_version=<?php echo self::VERSION; ?>&v=1" width="750" height="1500" frameborder="0"></iframe>
	</div>
	<?php
	}

	protected static function admin_products() {
		if (!current_user_can('manage_options'))  {
			wp_die( __('You do not have sufficient permissions to access this page.') );
		}
	?>	
	<div class="wrap">
		<h2>Products</h2>
		<table class="widefat post fixed" cellspacing="0">
			<thead>
				<tr>
					<th scope="col" id="name" width="5%" class="manage-column" style="">ID</th>
					<th scope="col" id="name" width="50%" class="manage-column" style="">Name</th>
					<th scope="col" id="cost" class="manage-column" style="">Cost</th>
					<th scope="col" id="downloads" class="manage-column num" style="">Downloads</th>
					<th scope="col" id="edit" class="manage-column num" style="">Edit</th>
					<th scope="col" id="delete" class="manage-column num" style="">Delete</th>
				</tr>
			</thead>

			<tfoot>
				<tr>
					<th scope="col" id="name" width="5%" class="manage-column" style="">ID</th>
					<th scope="col" id="name" class="manage-column" style="">Name</th>
					<th scope="col" id="cost" class="manage-column" style="">Cost</th>
					<th scope="col" id="downloads" class="manage-column num" style="">Downloads</th>
					<th scope="col" id="edit" class="manage-column num" style="">Edit</th>
					<th scope="col" id="delete" class="manage-column num" style="">Delete</th>
				</tr>
			</tfoot>

			<tbody>
				<?php
				global $wpdb;
				$table_name = $wpdb->prefix . "pfd_products";
				$products = $wpdb->get_results( "SELECT * FROM $table_name ORDER BY id ASC" ,ARRAY_A);
				if (count($products) == 0) {
				?>
				<tr class='alternate author-self status-publish iedit' valign="top">
					<td class="" colspan="5">No products. Fill out the form below to make one!</td>
				</tr>
				<?php
				} else {
				foreach ($products as $product) {
				?>
				<tr class='alternate author-self status-publish iedit' valign="top">
					<td class=""><?php echo $product['id'] ?></td>
					<td class="post-title column-title"><strong><a class="row-title" href="<?php echo get_option('siteurl') ?>/wp-admin/admin.php?page=paypal-file-download-products&action=edit&id=<?php echo $product['id'] ?>"><?php echo $product['name'] ?></a></strong></td>
					<td class=""><?php echo self::get_currency_symbol(); ?><?php echo $product['cost'] ?></td>
					<td class="" style="text-align:center;"><?php echo $product['downloads'] ?></td>
					<td class="" style="text-align:center;"><a href="<?php echo get_option('siteurl') ?>/wp-admin/admin.php?page=paypal-file-download-products&action=edit&id=<?php echo $product['id'] ?>">Edit</a></td>
					<td class="" style="text-align:center;"><a href="<?php echo get_option('siteurl') ?>/wp-admin/admin.php?page=paypal-file-download-products&action=delete&id=<?php echo $product['id'] ?>" onclick="if(confirm('Are you sure you want to delete this product? There is NO UNDO!')) { return true;} else { return false;}">Delete</a></td>
				</tr>
				<?php } } ?>
			</tbody>
		</table>
		
		<h2>Add a Product</h2>
		<form action="<?php echo get_option('siteurl') ?>/wp-admin/admin.php?page=paypal-file-download-products&action=add" method="post">
		<table class="form-table">
			<tr valign="top">
				<th scope="row">Product Name</th>
				<td><input type="text" name="product_name" style="width:250px;" value="" /></td>
			</tr>
			<tr valign="top">
				<th scope="row">URL of Product</th>
				<td><input type="text" name="product_url" style="width:400px;" value="" /><br />(make sure it's pretty secret.<br />This is never shown to the user. Instead, the file is streamed from this location directly to the user through a secret page, which helps to prevent content piracy.)</td>
			</tr>
			<tr valign="top">
				<th scope="row">Cost Per Download</th>
				<td><?php echo self::get_currency_symbol(); ?><input type="text" name="product_cost" style="width:50px;" value="" /></td>
			</tr>
			<tr valign="top">
				<th scope="row">&nbsp;</th>
				<td>
					<input type="submit" class="button-primary" value="Add Product" />
				</td>
			</tr>
		</table>
		</form>
	</div>
	<?php
	}

	protected static function admin_products_add() {
		// get shit
		$name = $_POST["product_name"];
		$url = $_POST["product_url"];
		$cost = $_POST["product_cost"];
		
		global $wpdb;
		$table_name = $wpdb->prefix . "pfd_products";
		
		$wpdb->insert( $table_name, array('name' => $name, 'file' => $url, 'cost' => $cost, 'downloads' => 0, 'created_at' => time()), array( '%s', '%s', '%s', '%d', '%d') );
		
		?>
		<script type="text/javascript">
		<!--
		window.location = "<?php echo get_option('siteurl') . '/wp-admin/admin.php?page=paypal-file-download-products' ?>"
		//-->
		</script>
		<?php
	}

	public static function admin_settings() {
		if (!current_user_can('manage_options'))  {
			wp_die( __('You do not have sufficient permissions to access this page.') );
		}
	?>
	<div class="wrap">
		<h2>PayPal File Download Settings</h2>
<?php
		if (isset($_GET['settings-updated'])) {
			echo '<div id="message" class="updated"><p>Updated settings!</p></div>';
		}
?>
		<form method="post" action="options.php">
			<?php settings_fields('pfd_options'); ?>
			<table class="form-table">
				<tr valign="top">
					<th scope="row">PayPal Email</th>
					<td><input type="text" name="paypal_email" style="width:300px;" value="<?php echo get_option('paypal_email'); ?>" /><br />This is the email transactions will post to. Make sure you have subscribed to PayPal Website Payments Standard.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Expire links after...</th>
					<td><input type="text" name="expire_links_after" style="width:50px;" value="<?php echo get_option('expire_links_after'); ?>" /> days (0 for no expiry)<br />Activating this will expire download links after a certain amount of time.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Return URL</th>
					<td><input type="text" name="paypal_return_url" style="width:250px;" value="<?php echo get_option('paypal_return_url'); ?>" /><br />This should be a URL (a page, preferably) on your site that says that the transaction has been completed and details (and download link) have been emailed to them.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Currency</th>
					<?php
						$cc = self::get_currency();
					?>
					<td>
						<select name="pfd_currency">
							<?php foreach(self::$currencies as $ccode => $currency) { ?>
							<option value="<?php echo $ccode; ?>" <?php if ($ccode == $cc) { ?>selected="selected"<?php } ?>><?php echo $currency[0]; ?> (<?php echo $currency[1]; ?>)</option>
							<?php } ?>
						</select>
						<br />Select the currency that your products will be sold in.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Email message</th>
					<td><textarea name="email_message" style="width:400px;height:200px;"><?php echo get_option('email_message'); ?></textarea><br />This will be what your customer receives after they pay.<br /><strong>The download link is automatically placed at the bottom of the message.</strong><br />You can use the following variables: [FIRST_NAME] [LAST_NAME] [DOWNLOAD_LINK] [PRODUCT_NAME] [TRANSACTION_ID]<br />Remember, your server must be set up to send mail.</td>
				</tr>
				<tr valign="top">
					<th scope="row">&nbsp;</th>
					<td>
						<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
					</td>
				</tr>
			</table>
		</form>
	</div>
	<?php
	}

	public static function admin_transactions() {
		if (!current_user_can('manage_options'))  {
			wp_die( __('You do not have sufficient permissions to access this page.') );
		}
	?>
	<div class="wrap">
		<h2>Transactions</h2>
		<table class="widefat post fixed" cellspacing="0">
			<thead>
				<tr>
					<th scope="col" id="name" width="30%" class="manage-column" style="">Transaction ID, Product</th>
					<th scope="col" id="name" width="20%" class="manage-column" style="">Date</th>
					<th scope="col" id="cost" width="6%" class="manage-column" style="">Cost</th>
					<th scope="col" id="name" width="15%" class="manage-column" style="">Name &amp; Email</th>
					<th scope="col" id="address" width="15%" class="manage-column" style="">Address</th>
					<th scope="col" id="fee" class="manage-column" style="">PayPal Fee</th>
					<th scope="col" id="earnings" class="manage-column" style="">Earnings</th>
				</tr>
			</thead>

			<tfoot>
				<tr>
					<th scope="col" id="name" width="30%" class="manage-column" style="">Transaction ID, Product</th>
					<th scope="col" id="name" width="20%" class="manage-column" style="">Date</th>
					<th scope="col" id="cost" width="6%" class="manage-column" style="">Cost</th>
					<th scope="col" id="name" width="15%" class="manage-column" style="">Name &amp; Email</th>
					<th scope="col" id="address" width="15%" class="manage-column" style="">Address</th>
					<th scope="col" id="fee" class="manage-column" style="">PayPal Fee</th>
					<th scope="col" id="earnings" class="manage-column" style="">Earnings</th>
				</tr>
			</tfoot>

			<tbody>
				<?php
				global $wpdb;
				$table_name = $wpdb->prefix . "pfd_transactions";
				$products_name = $wpdb->prefix . "pfd_products";
				$orders_name = $wpdb->prefix . "pfd_orders";
				
				$transactions = $wpdb->get_results( "SELECT $table_name.order_code, $products_name.name, $table_name.first_name, $table_name.created_at, $table_name.last_name, $table_name.address_street, $table_name.address_city, $table_name.address_state, $table_name.address_zip, $table_name.address_country, $table_name.payment_fee, $table_name.payer_email, $orders_name.cost FROM $table_name JOIN $products_name ON $table_name.product_id = $products_name.id JOIN $orders_name ON $table_name.order_id = $orders_name.id ORDER BY $table_name.id DESC" ,ARRAY_A);
				
				if (count($transactions) == 0) {
				?>
				<tr class='alternate author-self status-publish iedit' valign="top">
					<td class="" colspan="7">No transactions yet.</td>
				</tr>
				<?php
				} else {
				foreach ($transactions as $transaction) {
				?>
				<tr class='alternate author-self status-publish iedit' valign="top">
					<td class="post-title column-title"><?php echo $transaction['order_code'] ?><br /><strong><?php echo $transaction['name'] ?></strong></td>
					<td class=""><?php echo strftime("%a, %B %e, %Y %r", $transaction['created_at']) ?><br />(<?php echo self::relative_time($transaction["created_at"]) ?> ago)</td>
					<td class="">$<?php echo $transaction['cost'] ?></td>
					<td class=""><?php echo $transaction['first_name'] ?> <?php echo $transaction['last_name'] ?><br /><?php echo $transaction['payer_email'] ?></td>
					<td class=""><?php echo $transaction['address_street'] ?><br /><?php echo $transaction['address_city'] ?>, <?php echo $transaction['address_state'] ?> <?php echo $transaction['address_zip'] ?><br /><?php echo $transaction['address_country'] ?></td>
					<td class="">$<?php echo $transaction['payment_fee'] ?></td>
					<td class="">$<?php echo $transaction['cost'] - $transaction['payment_fee'] ?></td>
				</tr>
				
				<?php } } ?>
			</tbody>
		</table>
	</div>
	<?php
	}

	public static function media_button($context){
		$image_url = get_option('siteurl') . "/wp-content/plugins/paypal_file_download/media_icon.png";
		$more = '<a href="#TB_inline?width=450&inlineId=paypal_file_download_form" class="thickbox" title="Insert PayPal Checkout Link"><img src="' . $image_url . '" alt="Insert PayPal Checkout Link" /></a>';
		return $context . $more;
	}

	public static function add_pfd_form() {
	?>
	<script type="text/javascript">
		function insert_pfd_button(){
			product_id = jQuery("#product_selector").val()
			image = jQuery("#button_image_url").val()
			
			construct = '<a href="<?php echo get_option('siteurl') ?>/?checkout=' + product_id + '"><img src="' + image + '" /></a>';
			
			var wdw = window.dialogArguments || opener || parent || top;
			wdw.send_to_editor(construct);
		}

		function insert_pfd_link(){
			product_id = jQuery("#product_selector").val()
			
			construct = '<a href="<?php echo get_option('siteurl') ?>/?checkout=' + product_id + '"><?php echo get_option('siteurl') ?>/?checkout=' + product_id + '</a>';
			
			var wdw = window.dialogArguments || opener || parent || top;
			wdw.send_to_editor(construct);
		}
	</script>

	<div id="paypal_file_download_form" style="display:none;">
		<div class="wrap">
			<div>
				<div style="padding:15px 15px 0 15px;">
					<h3 style="font-size:16pt !important;line-height:1em !important;color:#555555 !important; font-family: Georgia, Times New Roman, Times, serif !important;"><span style="font-weight:normal; ">PayPal File Download</span><br />Insert Checkout Button/Link</h3>
					<span>Choose one of your products below to insert a download link to your page or post.</span>
				</div>
				<div style="padding:15px 15px 0 15px;">
					<table width="100%">
						<tr>
							<td width="135"><strong>Product</strong></td>
							<td>
								<select id="product_selector">
									<?php
									global $wpdb;
									$table_name = $wpdb->prefix . "pfd_products";
									$products = $wpdb->get_results( "SELECT * FROM $table_name ORDER BY id ASC;" ,ARRAY_A);
									if (count($products) == 0) {
									?>
									No products available. <a href="<?php echo get_option('siteurl') . "/wp-admin/admin.php?page=paypal-file-download-products" ?>">Save your document, then click here to add some.</a>
									<?php
									} else {
										foreach($products as $product) {
									?>
											<option value="<?php echo $product["id"] ?>"><?php echo $product["name"] ?> ($<?php echo $product["cost"] ?>)</option>
									<?php
									}
									}
									?>
								</select>
							</td>
						</tr>
						<tr>
							<td width="135"><strong>Button Image URL:</strong></td>
							<td><input type="text" id="button_image_url" value="https://www.paypal.com/en_US/i/btn/btn_buynowCC_LG.gif" style="width:220px;" /></td>
						</tr>
					</table>
				</div>
				
				<div style="padding:15px;">
					<input type="button" class="button-primary" value="Insert Button" onclick="insert_pfd_button();"/>&nbsp;&nbsp;&nbsp;&nbsp;<input type="button" class="button" value="Insert Link Only" onclick="insert_pfd_link();"/>&nbsp;&nbsp;&nbsp;&nbsp;<a style="font-size:0.9em;text-decoration:none;color:#555555;" href="#" onclick="tb_remove(); return false;">Cancel and Close</a>
				</div>
			</div>
		</div>
	</div>
	<?php
	}

	protected static function ipn() {
		echo "-- beginning ipn\n";
		require_once('paypal.class.php');
		$p = new paypal_class;
		$p->paypal_url = 'https://www.paypal.com/cgi-bin/webscr';
		
		$this_script = get_option('siteurl');
		
		echo "-- ipn validating...\n";
		
		if ($p->validate_ipn()) {
			echo "-- ipn validated\n";
			// find with order id
			global $wpdb;
			$table_name = $wpdb->prefix . "pfd_orders";
			
			$order = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE order_code = %s AND fulfilled = 0",$_POST["item_number"]) , ARRAY_A, 0);
			
			echo "-- found order\n";
			
			if ((string) $order["cost"] == $_POST["mc_gross"]) {
				// good
				echo "-- OK: costs match\n";
			} else {
				echo "-- NO: costs do not match. abort.\n";
				die("Error. Costs do not match.");
				// todo: send email
			}
			
			$wpdb->update( $table_name, array('fulfilled' => 1), array('id' => $order["id"]));
			
			echo "-- order marked as fulfilled\n";
			
			$table_name = $wpdb->prefix . "pfd_products";
			$product = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE id = %d",$order["product_id"]) , ARRAY_A, 0);
			
			$wpdb->update( $table_name, array('downloads' => $product["downloads"] + 1), array('id' => $product["id"]));
			
			$trans = array();
			
			// vars we want to extract
			$fields = "protection_eligibility address_status payer_id tax payment_date payment_status first_name last_name payer_status business address_name address_street address_city address_state address_zip address_country_code address_country quantity verify_sign payer_email txn_id payment_type receiver_email receiver_id txn_type item_name mc_currency item_number residence_country custom receipt_id transaction_subject payment_fee payment_gross";
			
			$fields_a = explode(" ",$fields);
			
			foreach($fields_a as $field) {
				$trans[$field] = isset($_POST[$field]) ? $_POST[$field] : NULL;
			}
			
			$trans["product_id"] = $product["id"];
			$trans["order_code"] = $order["order_code"];
			$trans["order_id"] = $order["id"];
			$trans["created_at"] = time();
			
			// insert into transactions
			$table_name = $wpdb->prefix . "pfd_transactions";
			$wpdb->insert($table_name, $trans);
			
			echo "-- transaction executed\n";
			
			echo "-- preparing email\n";
			
			// download link
			$download_link = get_option('siteurl') . "/?download=" . $_POST["item_number"];
			
			// get email text
			$emailtext = get_option('email_message');
			$emailtext = str_replace("[FIRST_NAME]",$_POST["first_name"],$emailtext);
			$emailtext = str_replace("[LAST_NAME]",$_POST["last_name"],$emailtext);
			$emailtext = str_replace("[DOWNLOAD_LINK]",$download_link,$emailtext);
			$emailtext = str_replace("[PRODUCT_NAME]",$product["name"],$emailtext);
			$emailtext = str_replace("[TRANSACTION_ID]",$_POST["item_number"],$emailtext);
			
			$emailtext = $emailtext . "\n\nYou can download your file here:\n" . $download_link;
			
			// fantastic, now send them a message
			$to      = $_POST["payer_email"];
			
			$subject = "Receipt and Download for your purchase of " . $product["name"];
			$message = $emailtext;
			
			$headers = 'From: '. get_option('blogname') .' <' . get_option('admin_email') . '>' . "\r\n Reply-To: " . get_option('admin_email') . "\r\n X-Mailer: PayPalFileDownload";
			
			wp_mail($to, $subject, $message, $headers);
			
			echo "-- email sent to " . $to . "\n";
		}
	}


	public static function var_listener() {
		// listen for IPN
		//if (get_query_var("pfd_action") == "ipn") {
			// IPN activated
		//	self::ipn();
		if(get_query_var("checkout")==NULL) {
			// no checkout variable, proceed to check for download
			if(get_query_var("download")==NULL) {
				// no download variable, proceed to check for IPN
				if (get_query_var("pfd_action") == "ipn") {
					// IPN activated
					self::ipn();

					// don't execute wp
					exit();
				}
			} else {
				// download
				// get ID
				$id = $_GET["download"];
				
				global $wpdb;

				$table_name = $wpdb->prefix . "pfd_transactions";
				$transaction = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE order_code = %s",$id ), ARRAY_A, 0);
				
				if ($transaction==NULL) {
					die("File not found.");
				} else {
					$table_name = $wpdb->prefix . "pfd_products";
					$product = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE id = %d",$transaction["product_id"]), ARRAY_A, 0);
					
					// get option for days
					$daysexpire = get_option('expire_links_after');
					if ($daysexpire == 0) {
						// don't check
					} else {
						// check for expiry
						// transaction created at should be larger than now - x days
						$nowminus = time() - ($daysexpire*86400);
						if ($transaction["created_at"] > $nowminus) {
							// good
						} else {
							die("Sorry, your download has expired.");
						}
					}
					
					// force download
					header('Content-disposition: attachment; filename=' . basename($product["file"]));
					header('Content-Type: application/octet-stream');
					header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
					header('Expires: 0');
					$result = wp_remote_get($product["file"]);
					echo $result['body'];
					die();
				}
			}
		} else {
			// oh no!
			// get ID
			$product_id = get_query_var("checkout");
			
			global $wpdb;
			$table_name = $wpdb->prefix . "pfd_products";
			
			// get product
			$product = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE id = %d",$product_id) , ARRAY_A, 0);
			
			// construct order
			$table_name = $wpdb->prefix . "pfd_orders";
			
			$t_code = self::transactioncode();
			
			$wpdb->insert( $table_name, array('product_id' => $product_id, 'order_code' => $t_code, 'fulfilled' => 0, 'created_at' => time(), 'cost' => $product["cost"]), array( '%d', '%s', '%d', '%d', '%s') );
			
			$ccode = self::get_currency();
			
			// get url for paypal
			$paypal_url = "https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=" . get_option('paypal_email') . "&item_name=" . urlencode($product["name"]) . "&item_number=" . $t_code . "&amount=" . urlencode($product["cost"]) . "&currency_code=" . $ccode . "&notify_url=" . urlencode(get_option('siteurl') . "/?pfd_action=ipn");
			
			$paypal_url = $paypal_url . "&return=" . urlencode(get_option('paypal_return_url'));
			
			header('Location: ' . $paypal_url);
			
			exit; // don't execute wp
		}
		
	}

	// make sure we have the paypal action listener available
	public static function register_vars($vars) {
		$vars[] = "pfd_action";
		$vars[] = "checkout";
		$vars[] = "download";
		return $vars; // return to wordpress
	}
}